"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.builder = exports.command = exports.describe = exports.handler = exports.exportEventStore = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const std_1 = require("../../utils/std");
const command_handler_1 = __importDefault(require("../../command-handler"));
const progress_bar_1 = __importDefault(require("../../utils/progress-bar"));
const fetch_1 = __importDefault(require("../../utils/fetch"));
const exportEventStore = async (params) => {
    const { eventStorePath, eventStoreId, client } = params;
    const pathToEventStore = path_1.default.resolve(process.cwd(), eventStorePath);
    if (!fs_1.default.existsSync(pathToEventStore)) {
        fs_1.default.mkdirSync(pathToEventStore);
    }
    const pathToEvents = path_1.default.join(pathToEventStore, 'events.db');
    const pathToSecrets = path_1.default.join(pathToEventStore, 'secrets.db');
    const { eventsExportUrl, secretsExportUrl, statusFileUrl } = await client.getExportUrls({
        eventStoreId,
    });
    await client.exportEvents({
        eventStoreId,
    });
    const interval = 16 * 60 * 1000; // 16 minutes
    let progressBar;
    let prevUploadedEvents = 0;
    let startTime = Date.now();
    let skipExport = false;
    for (;;) {
        try {
            // eslint-disable-next-line no-loop-func
            const isComplete = await new Promise(async (resolve, reject) => {
                try {
                    const res = await (0, fetch_1.default)(statusFileUrl, {
                        method: 'GET',
                    }, [404]);
                    if (res.status === 404) {
                        resolve(false);
                        return;
                    }
                    const { eventsUploaded, eventCount, heartbeatTime } = await res.json();
                    if (eventCount === 0) {
                        skipExport = true;
                        return resolve(true);
                    }
                    if (progressBar == null) {
                        progressBar = new progress_bar_1.default('  preparing event-store [:bar] :percent', {
                            width: 50,
                            total: eventCount,
                        });
                        progressBar.render();
                    }
                    if (prevUploadedEvents !== eventsUploaded) {
                        progressBar.tick(eventsUploaded - prevUploadedEvents);
                        prevUploadedEvents = eventsUploaded;
                        startTime = heartbeatTime;
                    }
                    if (Date.now() - heartbeatTime > interval) {
                        reject(new Error('Timeout'));
                        return;
                    }
                    if (eventsUploaded >= eventCount) {
                        resolve(true);
                    }
                    else {
                        resolve(false);
                    }
                }
                catch (error) {
                    reject(error);
                }
            });
            if (Date.now() - startTime > interval) {
                throw new Error('Timeout');
            }
            if (isComplete) {
                break;
            }
            await new Promise((resolve) => setTimeout(resolve, 5000));
        }
        catch (error) {
            console.log(error);
            throw error;
        }
    }
    if (skipExport) {
        return;
    }
    await Promise.all([
        {
            type: 'events',
            filePath: pathToEvents,
            exportUrl: eventsExportUrl,
        },
        {
            type: 'secrets',
            filePath: pathToSecrets,
            exportUrl: secretsExportUrl,
        },
    ].map(({ type, exportUrl, filePath }) => {
        const callback = async (resolve, reject) => {
            try {
                const res = await (0, fetch_1.default)(exportUrl, {
                    method: 'GET',
                });
                const fileStream = fs_1.default.createWriteStream(filePath);
                await new Promise(async (_resolve, _reject) => {
                    res.body.on('error', _reject);
                    fileStream.on('finish', _resolve);
                    if (type === 'events') {
                        const size = res.headers.get('content-length');
                        if (size != null) {
                            const bar = new progress_bar_1.default(`  downloading event-store [:bar] :percent`, {
                                width: 50,
                                total: +size,
                            });
                            for await (const chunk of res.body) {
                                fileStream.write(chunk);
                                bar.tick(chunk.length);
                            }
                        }
                    }
                    else {
                        res.body.pipe(fileStream);
                    }
                });
                resolve();
            }
            catch (error) {
                reject(error);
            }
        };
        return new Promise(callback).catch(async (error) => {
            void error;
            await new Promise((resolve) => setTimeout(resolve, 5000));
            return new Promise(callback);
        });
    }));
};
exports.exportEventStore = exportEventStore;
exports.handler = (0, command_handler_1.default)(async ({ client }, params) => {
    const { path: eventStorePath, 'event-store-id': eventStoreId } = params;
    std_1.logger.start(`exporting the event-store from the cloud`);
    await (0, exports.exportEventStore)({
        client,
        eventStorePath,
        eventStoreId,
    });
    std_1.logger.success('Export event-store successfully completed!');
});
exports.describe = chalk_1.default.green('export an event store from the cloud');
exports.command = 'export <event-store-id> <path>';
const builder = (yargs) => yargs
    .positional('event-store-id', {
    describe: chalk_1.default.green("an existing event store's id"),
    type: 'string',
})
    .positional('path', {
    describe: chalk_1.default.green('path to the event store directory'),
    type: 'string',
});
exports.builder = builder;
//# sourceMappingURL=export.js.map