"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.builder = exports.command = exports.describe = exports.handler = exports.importEventStore = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const readline_1 = __importDefault(require("readline"));
const std_1 = require("../../utils/std");
const progress_bar_1 = __importDefault(require("../../utils/progress-bar"));
const fetch_1 = __importDefault(require("../../utils/fetch"));
const command_handler_1 = __importDefault(require("../../command-handler"));
const PART_SIZE = 100 * 1024 * 1024; // 100 MB in Bytes
const eventFieldsQueue = [
    'threadId',
    'threadCounter',
    'timestamp',
    'aggregateId',
    'aggregateVersion',
    'type',
    'payload',
    'eventSize',
];
const secretFieldsQueue = ['idx', 'id', 'secret'];
const escapeCsv = (str) => `"${str.replace(/"/g, '""')}"`;
const getPathToEvents = (eventStorePath) => path_1.default.resolve(process.cwd(), eventStorePath, 'events.db');
const getPathToSecrets = (eventStorePath) => path_1.default.resolve(process.cwd(), eventStorePath, 'secrets.db');
async function* generator(filePath, type, size, format) {
    const readStream = fs_1.default.createReadStream(filePath);
    const readByLine = readline_1.default.createInterface(readStream);
    if (type === 'events') {
        const bar = new progress_bar_1.default(`  preparing event-store [:bar] :percent`, {
            width: 50,
            total: size,
            format,
        });
        readStream.on('data', (chunk) => {
            bar.tick(chunk.length);
        });
    }
    for await (const line of readByLine) {
        const eventStoreItem = JSON.parse(line);
        const chunk = filePath.includes('events.db')
            ? eventFieldsQueue
                .map((key) => {
                if (key === 'payload') {
                    return escapeCsv(JSON.stringify(eventStoreItem[key]));
                }
                if (key === 'aggregateId' || key === 'type') {
                    return escapeCsv(eventStoreItem[key]);
                }
                if (key === 'eventSize' && eventStoreItem[key] == null) {
                    const serializedEvent = [
                        `"${eventStoreItem.aggregateId}",`,
                        `${+eventStoreItem.aggregateVersion},`,
                        `"${eventStoreItem.type}",`,
                        `"${JSON.stringify(eventStoreItem.payload != null ? eventStoreItem.payload : null)}"`,
                    ].join('');
                    return Buffer.byteLength(serializedEvent) + 66;
                }
                return eventStoreItem[key];
            })
                .join(',')
            : secretFieldsQueue
                .map((key) => {
                if (key === 'id' || key === 'secret') {
                    return escapeCsv(eventStoreItem[key]);
                }
                return eventStoreItem[key];
            })
                .join(',');
        yield `${chunk}\n`;
    }
}
const generateCsv = async (eventStorePath, format) => {
    const pathToEvents = getPathToEvents(eventStorePath);
    const pathToSecrets = getPathToSecrets(eventStorePath);
    return (await Promise.all([
        {
            type: 'events',
            originalFilePath: pathToEvents,
        },
        {
            type: 'secrets',
            originalFilePath: pathToSecrets,
        },
    ].map(({ originalFilePath, type }) => new Promise(async (resolve, reject) => {
        const { size } = fs_1.default.statSync(originalFilePath);
        const readStream = generator(originalFilePath, type, size, format);
        let chunkSize = 0;
        let part = 0;
        const pathsToCsv = [path_1.default.resolve(process.cwd(), eventStorePath, `${type}-${part}.csv`)];
        let writeStream = fs_1.default.createWriteStream(pathsToCsv[part]);
        try {
            for await (const line of readStream) {
                chunkSize += Buffer.byteLength(line);
                if (chunkSize >= PART_SIZE) {
                    chunkSize = 0;
                    part += 1;
                    pathsToCsv.push(path_1.default.resolve(process.cwd(), eventStorePath, `${type}-${part}.csv`));
                    writeStream = fs_1.default.createWriteStream(pathsToCsv[part]);
                }
                writeStream.write(line);
            }
            resolve({
                type,
                partCount: part + 1,
                pathsToCsv,
            });
        }
        catch (error) {
            reject(error);
        }
    }))));
};
const deleteCsv = async (items) => {
    for (const { pathsToCsv } of items) {
        for (const filePath of pathsToCsv) {
            if (fs_1.default.existsSync(filePath)) {
                fs_1.default.unlinkSync(filePath);
            }
        }
    }
};
const importEventStore = async (params) => {
    const { eventStorePath, eventStoreId, client, format } = params;
    const pathToEvents = getPathToEvents(eventStorePath);
    const pathToSecrets = getPathToSecrets(eventStorePath);
    if (!fs_1.default.existsSync(pathToEvents)) {
        throw new Error(`No such file or directory "${pathToEvents}"`);
    }
    if (!fs_1.default.existsSync(pathToSecrets)) {
        throw new Error(`No such file or directory "${pathToSecrets}"`);
    }
    const generatedFiles = await generateCsv(eventStorePath, format);
    try {
        const eventsPartsInfo = generatedFiles.find(({ type }) => type === 'events');
        const secretsPartsInfo = generatedFiles.find(({ type }) => type === 'secrets');
        if (eventsPartsInfo == null) {
            throw new Error('Failed to prepare events for import');
        }
        if (secretsPartsInfo == null) {
            throw new Error('Failed to prepare secrets for import');
        }
        const { eventsImportUrls, secretsImportUrls } = await client.getImportUrls({
            eventStoreId,
            eventsPartCount: eventsPartsInfo.partCount,
            secretsPartCount: secretsPartsInfo.partCount,
        });
        const uploadingBar = new progress_bar_1.default(`  uploading event-store [:bar] :percent`, {
            width: 50,
            total: eventsPartsInfo.partCount + secretsPartsInfo.partCount,
            format,
        });
        std_1.logger.debug(`upload events and secrets`);
        try {
            await Promise.all([
                ...eventsImportUrls.map((url, idx) => ({
                    uploadUrl: url,
                    filePath: eventsPartsInfo.pathsToCsv[idx],
                })),
                ...secretsImportUrls.map((url, idx) => ({
                    uploadUrl: url,
                    filePath: secretsPartsInfo.pathsToCsv[idx],
                })),
            ].map(({ filePath, uploadUrl }) => new Promise(async (resolve, reject) => {
                const { size: fileSizeInBytes } = fs_1.default.statSync(filePath);
                if (fileSizeInBytes === 0) {
                    uploadingBar.tick();
                    return resolve();
                }
                const contentType = 'text/csv';
                const fileStream = fs_1.default.createReadStream(filePath);
                try {
                    await (0, fetch_1.default)(uploadUrl, {
                        method: 'PUT',
                        headers: {
                            'Content-Length': fileSizeInBytes.toString(),
                            'Content-Type': contentType,
                        },
                        body: fileStream,
                    });
                    uploadingBar.tick();
                    resolve();
                }
                catch (error) {
                    reject(error);
                }
            })));
            std_1.logger.debug(`events and secrets have been uploaded`);
        }
        catch (error) {
            std_1.logger.debug(`failed to upload events and secrets`);
            throw new Error(error);
        }
        const totalPartCount = Math.max(eventsPartsInfo.partCount, secretsPartsInfo.partCount);
        const importingBar = new progress_bar_1.default(`  importing event-store [:bar] :percent`, {
            width: 50,
            total: totalPartCount,
            format,
        });
        importingBar.render();
        for (let partIndex = 0; partIndex < totalPartCount; partIndex++) {
            try {
                await client.importEvents({
                    eventStoreId,
                    partIndex,
                });
                importingBar.tick();
            }
            catch (error) {
                if (/Unable to complete the COPY from S3/.test(`${error}`)) {
                    partIndex--;
                }
                else {
                    throw error;
                }
            }
        }
    }
    finally {
        await deleteCsv(generatedFiles);
    }
};
exports.importEventStore = importEventStore;
exports.handler = (0, command_handler_1.default)(async ({ client }, params) => {
    const { path: eventStorePath, 'event-store-id': eventStoreId } = params;
    std_1.logger.start(`importing the event-store to the cloud`);
    await (0, exports.importEventStore)({
        client,
        eventStorePath,
        eventStoreId,
    });
    std_1.logger.success('Import event-store successfully completed!');
});
exports.describe = chalk_1.default.green('import an event-store to the cloud');
exports.command = 'import <event-store-id> <path>';
const builder = (yargs) => yargs
    .positional('event-store-id', {
    describe: chalk_1.default.green("an existing event store's id"),
    type: 'string',
})
    .positional('path', {
    describe: chalk_1.default.green('path to the event-store directory'),
    type: 'string',
});
exports.builder = builder;
//# sourceMappingURL=import.js.map