"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const config_1 = require("../config");
const std_1 = require("../utils/std");
const readdir = (0, util_1.promisify)(fs_1.default.readdir);
const lstat = (0, util_1.promisify)(fs_1.default.lstat);
const readFile = (0, util_1.promisify)(fs_1.default.readFile);
const collectWorkspacePackages = async (root) => {
    const checkCandidate = async (candidate) => {
        try {
            const packageJson = JSON.parse((await readFile(path_1.default.resolve(candidate.path, 'package.json'))).toString());
            return {
                ...candidate,
                name: packageJson.name,
                valid: true,
            };
        }
        catch (e) {
            std_1.logger.warn(`suspicious package located at ${candidate.path}`);
            return { ...candidate, valid: false };
        }
    };
    const onionTest = /^(.*)\*$/i.exec(root);
    if (onionTest) {
        const actualRoot = onionTest[1];
        if (actualRoot == null) {
            return {};
        }
        return readdir(actualRoot)
            .then((entries) => Promise.all(entries.map(async (entry) => {
            const directory = path_1.default.resolve(actualRoot, entry);
            const stats = await lstat(directory);
            return {
                path: directory,
                isDirectory: stats.isDirectory(),
            };
        })))
            .then((entries) => entries.filter((entry) => entry.isDirectory))
            .then((candidates) => Promise.all(candidates.map(checkCandidate)))
            .then((candidates) => candidates.filter((candidate) => candidate.valid))
            .then((subPackages) => subPackages.reduce((map, item) => {
            map[item.name] = item.path;
            return map;
        }, {}));
    }
    const candidate = await checkCandidate({
        path: root,
    });
    if (candidate.valid) {
        return {
            [candidate.name]: candidate.path,
        };
    }
};
const collectPackages = async (root) => {
    try {
        const frameworkJson = JSON.parse(fs_1.default.readFileSync(path_1.default.resolve(root, 'package.json')).toString());
        const allowedLocations = frameworkJson.workspaces.packages.filter((location) => location.startsWith('packages/'));
        return Promise.all(allowedLocations.map((location) => collectWorkspacePackages(path_1.default.resolve(root, location)))).then((result) => result.reduce((map, entry) => Object.assign(map, entry), {}));
    }
    catch (e) {
        std_1.logger.error(e.message);
        return {};
    }
};
const copyPackages = async (cloudEntry, frameworkPackages) => {
    try {
        const dependencies = await readdir(path_1.default.resolve(cloudEntry, 'node_modules')).then((entries) => entries.filter((entry) => Object.prototype.hasOwnProperty.call(frameworkPackages, entry)));
        await Promise.all(dependencies.map(async (name) => {
            const targetDirectory = path_1.default.resolve(cloudEntry, 'node_modules', name);
            std_1.logger.trace(`[experimental] linking package ${name} -> ${targetDirectory}`);
            await fs_extra_1.default.emptyDir(targetDirectory);
            await fs_extra_1.default.copy(frameworkPackages[name], targetDirectory, {
                overwrite: true,
                dereference: true,
            });
        }));
        return 0;
    }
    catch (e) {
        std_1.logger.error(e);
        return 1;
    }
};
exports.default = async (cloudEntry) => {
    const resolveSymlink = (0, config_1.get)('symlinks.resolve');
    const linkedProjects = (0, config_1.get)('linked_projects') || [];
    const projectDirectory = path_1.default.resolve();
    if (resolveSymlink && linkedProjects.find((directory) => directory === projectDirectory)) {
        std_1.logger.trace('[experimental] injecting linked framework to target node_modules');
        return copyPackages(cloudEntry, await collectPackages(resolveSymlink));
    }
    return 0;
};
//# sourceMappingURL=symlinks.js.map